package test.avalara.services;

import junit.framework.*;
import com.avalara.avatax.services.tax.*;
import com.avalara.avatax.services.base.*;

import java.io.*;
import javax.xml.rpc.ServiceException;
import javax.xml.soap.SOAPException;
import java.rmi.RemoteException;
import java.util.Date;
import java.util.Properties;
import java.util.Calendar;
import java.math.BigDecimal;
import java.net.URL;
import java.net.MalformedURLException;


/**
 * JUnit tests for Avalara AvaTax Tax
 */
public class TaxSvcTest extends TestCase
{
    private TaxSvc taxSvc;
    private TaxSvcSoap port;


    public static void main(String args[]) {
        junit.textui.TestRunner.run(TaxSvcTest.class);
    }
    protected void setUp() throws ServiceException, SOAPException, MalformedURLException, IOException
    {
        Properties properties = new java.util.Properties();
        File file = new File("avatax4j.properties");
        try
        {
            properties.load(new FileInputStream(file));
        } catch (IOException e)
        {
            System.out.println("Unable to load avatax4j.properties");
            throw e;
        }

        String url = properties.getProperty("avatax4j.url");
        Assert.assertFalse("avatax4j.url property is not configured in avatax4j.properties", url == null || url.length() == 0);

        String account = properties.getProperty("avatax4j.account");
        Assert.assertFalse("avatax4j.account property is not configured in avatax4j.properties", account == null || account.length() == 0);

        String license = properties.getProperty("avatax4j.license");
        Assert.assertFalse("avatax4j.license property is not configured in avatax4j.properties", license == null || license.length() == 0);

        taxSvc = new TaxSvcLocator();

        port = taxSvc.getTaxSvcSoap(new URL(url));

        // Set the profile
        Profile profile = new Profile();
        profile.setName("5.2.0.0");
        profile.setClient("TaxSvcTest,5.2.0.0");
        port.setProfile(profile);

        // Set security
        Security security = new Security();
        security.setAccount(account);
        security.setLicense(license);
        port.setSecurity(security);
    }

    public void testPing() throws RemoteException
    {
        PingResult result = port.ping("");

        Assert.assertEquals(SeverityLevel.Success, result.getResultCode());
    }

    public void testIsAuthorized() throws RemoteException
    {
        IsAuthorizedResult result = port.isAuthorized("Ping,IsAuthorized,GetTax,PostTax,CommitTax,CancelTax,ReconcileTaxHistory");

        Assert.assertEquals(SeverityLevel.Success, result.getResultCode());
        Assert.assertEquals("Ping,IsAuthorized,GetTax,PostTax,CommitTax,CancelTax,ReconcileTaxHistory", result.getOperations());
    }

    public void testGetTaxDocumentDetail() throws RemoteException
    {
        GetTaxRequest request = CreateTaxRequest("GetTaxDocumentDetailTest");

        GetTaxResult result = port.getTax(request);

        Assert.assertEquals(SeverityLevel.Success, result.getResultCode());
        Assert.assertEquals(DocStatus.Saved, result.getDocStatus());
        Assert.assertFalse(result.isReconciled());
        Assert.assertEquals(new BigDecimal(1010), result.getTotalAmount());
        Assert.assertEquals(BigDecimal.valueOf(9595, 2), result.getTotalTax());

        // Check tax history
        GetTaxHistoryRequest taxHistoryRequest = new GetTaxHistoryRequest();
        taxHistoryRequest.setDocId(result.getDocId());
        taxHistoryRequest.setDetailLevel(DetailLevel.Tax);
        GetTaxHistoryResult taxHistoryResult = port.getTaxHistory(taxHistoryRequest);

        Assert.assertEquals(SeverityLevel.Success, taxHistoryResult.getResultCode());
        compareHistory(request, result, taxHistoryResult);
    }

    public void testGetTaxRounding() throws RemoteException
    {
        GetTaxRequest request = CreateTaxRequest("GetTaxDocumentDetailTest-Java");
        request.setDetailLevel(DetailLevel.Tax);
        request.getLines().getLine(0).setAmount(new BigDecimal(4599));
        request.getLines().getLine(1).setAmount(new BigDecimal(50));

        GetTaxResult result = port.getTax(request);

        Assert.assertEquals(SeverityLevel.Success, result.getResultCode());
        Assert.assertEquals(BigDecimal.valueOf(44166, 2), result.getTotalTax());

        // Check tax history
        GetTaxHistoryRequest taxHistoryRequest = new GetTaxHistoryRequest();
        taxHistoryRequest.setDocId(result.getDocId());
        taxHistoryRequest.setDetailLevel(DetailLevel.Tax);
        GetTaxHistoryResult taxHistoryResult = port.getTaxHistory(taxHistoryRequest);

        Assert.assertEquals(SeverityLevel.Success, taxHistoryResult.getResultCode());
        compareHistory(request, result, taxHistoryResult);
    }



    /*
    public void GetTaxRoundingTest()
    {
        GetTaxRequest request = CreateTaxRequest("GetTaxRoundingTest");
        request.Lines[0].Amount = 4599m;
        request.Lines[1].Amount = 50m;
        request.DetailLevel = DetailLevel.Tax;
        // Swap origin and dest addresses
        request.Addresses[0].AddressCode = "Dest";
        request.Addresses[1].AddressCode = "Origin";

        GetTaxResult result = svc.GetTax(request);

        Assert.AreEqual(SeverityLevel.Success, result.ResultCode, FormatMessages(result));

        decimal totalTax = 0m;
        decimal totalRate = 0m;
        foreach (TaxDetail detail in result.TaxLines[0].TaxDetails)
        {
            totalTax += detail.Tax;
            totalRate += detail.Rate;
        }

        Assert.AreEqual(404.712m, totalTax, "TotalTax expected");
        Assert.AreEqual(Round(totalTax, 2), result.TaxLines[0].Tax, "TotalTax compare");
        Assert.AreEqual(totalRate, result.TaxLines[0].Rate, "TotalRate compare");

        // Verify history
        GetTaxHistoryRequest historyRequest = new GetTaxHistoryRequest();
        historyRequest.CompanyCode = request.CompanyCode;
        historyRequest.DetailLevel = DetailLevel.Line;
        historyRequest.DocCode = request.DocCode;
        historyRequest.DocType = request.DocType;
        historyRequest.DetailLevel = DetailLevel.Tax;

        GetTaxHistoryResult historyResult = svc.GetTaxHistory(historyRequest);
        Assert.AreEqual(SeverityLevel.Success, historyResult.ResultCode, FormatMessages(historyResult));
        CompareHistory(request, result, historyResult);
    }
    */

    public void testTransaction() throws RemoteException
    {
        Date date = new Date();
        String docCode = "CommitTest " + Long.toString(date.getTime());
        GetTaxRequest taxRequest = CreateTaxRequest(docCode);
        //taxRequest.setDetailLevel(DetailLevel.Tax);
        taxRequest.setDetailLevel(DetailLevel.Diagnostic);

        GetTaxResult taxResult = port.getTax(taxRequest);

        Assert.assertEquals(SeverityLevel.Success, taxResult.getResultCode());
        Assert.assertEquals(taxRequest.getLines().size(), taxResult.getTaxLines().size());
        TaxLine taxLine = taxResult.getTaxLines().getTaxLine(0);
        Assert.assertEquals(2, taxLine.getTaxDetails().size());             
       
        TaxDetail taxDetail = taxLine.getTaxDetails().getTaxDetail(0);
        Assert.assertEquals(JurisdictionType.State, taxDetail.getJurisType());
        Assert.assertEquals("53", taxDetail.getJurisCode());
        Assert.assertEquals(BigDecimal.valueOf(1000), taxDetail.getTaxable());
        Assert.assertEquals(BigDecimal.valueOf(65000,6), taxDetail.getRate());
        Assert.assertEquals("G", taxDetail.getRateType());
        Assert.assertEquals(BigDecimal.valueOf(65), taxDetail.getTax());
        Assert.assertEquals(BigDecimal.valueOf(0), taxDetail.getNonTaxable());
        Assert.assertEquals(taxDetail.getTaxable(), taxDetail.getTaxable());

        // Check tax history
        GetTaxHistoryRequest taxHistoryRequest = new GetTaxHistoryRequest();
        taxHistoryRequest.setDocId(taxResult.getDocId());
        //taxHistoryRequest.setDetailLevel(DetailLevel.Tax);
        taxHistoryRequest.setDetailLevel(DetailLevel.Diagnostic);
        GetTaxHistoryResult taxHistoryResult = port.getTaxHistory(taxHistoryRequest);

        Assert.assertEquals(SeverityLevel.Success, taxHistoryResult.getResultCode());
        GetTaxRequest historyTaxRequest = taxHistoryResult.getGetTaxRequest();
        Assert.assertNotNull(historyTaxRequest);
        Assert.assertEquals(taxRequest.getAddresses().size(), historyTaxRequest.getAddresses().size());
        Assert.assertEquals(taxRequest.getLines().size(), historyTaxRequest.getLines().size());
        Assert.assertEquals(0,taxRequest.getExchangeRate().compareTo(taxHistoryResult.getGetTaxRequest().getExchangeRate()));
        //Assert.assertEquals(taxRequest.getExchangeRateEffDate(),taxHistoryResult.getGetTaxRequest().getExchangeRateEffDate());

        compareHistory(taxRequest,taxResult,taxHistoryResult);

        GetTaxResult historyTaxResult = taxHistoryResult.getGetTaxResult();
        Assert.assertNotNull(historyTaxResult);
        Assert.assertEquals(taxRequest.getLines().size(), historyTaxResult.getTaxLines().size());

        PostTaxRequest postRequest = new PostTaxRequest();

        postRequest.setCompanyCode(taxRequest.getCompanyCode());
        postRequest.setDocType(taxRequest.getDocType());
        postRequest.setDocCode(taxRequest.getDocCode());
        postRequest.setDocDate(taxRequest.getDocDate());
        postRequest.setTotalAmount(taxResult.getTotalAmount());
        postRequest.setTotalTax(taxResult.getTotalTax());

        PostTaxResult postResult = port.postTax(postRequest);

        Assert.assertEquals(SeverityLevel.Success, postResult.getResultCode());

        CommitTaxRequest commitRequest = new CommitTaxRequest();
        commitRequest.setDocId(postResult.getDocId());
        commitRequest.setDocType(taxRequest.getDocType());

        CommitTaxResult commitResult = port.commitTax(commitRequest);

        Assert.assertEquals(SeverityLevel.Success, commitResult.getResultCode());

        CancelTaxRequest cancelRequest = new CancelTaxRequest();
        cancelRequest.setDocId(commitResult.getDocId());
        cancelRequest.setDocType(taxRequest.getDocType());
        cancelRequest.setCancelCode(CancelCode.DocDeleted);

        CancelTaxResult cancelResult = port.cancelTax(cancelRequest);

        Assert.assertEquals(SeverityLevel.Success, cancelResult.getResultCode());

    }

    public void testReconcileHistoryTest() throws RemoteException
    {
        Date date = new Date();
        String docCode = "CommitTest " + Long.toString(date.getTime());
        GetTaxRequest taxRequest = CreateTaxRequest(docCode);
        taxRequest.setDetailLevel(DetailLevel.Tax);

        GetTaxResult taxResult = port.getTax(taxRequest);

        Assert.assertEquals(SeverityLevel.Success, taxResult.getResultCode());
        Assert.assertEquals(taxRequest.getLines().size(), taxResult.getTaxLines().size());
        TaxLine taxLine = taxResult.getTaxLines().getTaxLine(0);
        Assert.assertEquals(taxLine.getTaxDetails().size(),2);
        TaxDetail taxDetail = taxLine.getTaxDetails().getTaxDetail(0);
        Assert.assertEquals(JurisdictionType.State, taxDetail.getJurisType());
        Assert.assertEquals("53", taxDetail.getJurisCode());
        Assert.assertEquals(BigDecimal.valueOf(1000), taxDetail.getTaxable());
        Assert.assertEquals(BigDecimal.valueOf(65000,6), taxDetail.getRate());
        Assert.assertEquals(BigDecimal.valueOf(65), taxDetail.getTax());
        Assert.assertEquals(BigDecimal.valueOf(0), taxDetail.getNonTaxable());
        Assert.assertEquals(taxDetail.getTaxable(), taxDetail.getTaxable());

        PostTaxRequest postRequest = new PostTaxRequest();

        postRequest.setCompanyCode(taxRequest.getCompanyCode());
        postRequest.setDocType(taxRequest.getDocType());
        postRequest.setDocCode(taxRequest.getDocCode());
        postRequest.setDocDate(taxRequest.getDocDate());
        postRequest.setTotalAmount(taxResult.getTotalAmount());
        postRequest.setTotalTax(taxResult.getTotalTax());

        PostTaxResult postResult = port.postTax(postRequest);

        Assert.assertEquals(SeverityLevel.Success, postResult.getResultCode());

        CommitTaxRequest commitRequest = new CommitTaxRequest();
        commitRequest.setDocId(postResult.getDocId());
        commitRequest.setDocType(taxRequest.getDocType());

        CommitTaxResult commitResult = port.commitTax(commitRequest);

        Assert.assertEquals(SeverityLevel.Success, commitResult.getResultCode());

        //Preparing RecontaxHistory Request
        ReconcileTaxHistoryRequest request = new ReconcileTaxHistoryRequest();
        request.setCompanyCode("DEFAULT");
        request.setReconciled(false);
        request.setStartDate(new Date());
        request.setEndDate(new Date());
        request.setDocStatus(DocStatus.Committed);

        //Calling reconHistory Method
        ReconcileTaxHistoryResult result = port.reconcileTaxHistory(request);
        GetTaxResult taxResults[] = result.getGetTaxResults().getGetTaxResult();
        //checking the results
        Assert.assertEquals(SeverityLevel.Success, result.getResultCode());
        //checking if we get the same DocId

        boolean found = false;
        for (int ii = 0; ii < taxResults.length && !found; ii++)
        {
            if (taxResult.getDocId().equals(taxResults[ii].getDocId()))
            {
                found = true;
            }
        }
        Assert.assertTrue("Document not found in reconcile results", found);

        CancelTaxRequest cancelRequest = new CancelTaxRequest();
        cancelRequest.setDocId(commitResult.getDocId());
        cancelRequest.setDocType(taxRequest.getDocType());
        cancelRequest.setCancelCode(CancelCode.DocDeleted);

        CancelTaxResult cancelResult = port.cancelTax(cancelRequest);

        Assert.assertEquals(SeverityLevel.Success, cancelResult.getResultCode());

    }

    public void testApplyPayment() throws RemoteException
    {

        Date date=new Date();
        Calendar calendar=Calendar.getInstance();

        GetTaxRequest request = CreateTaxRequest("ApplyPaymentTest");
        request.setDetailLevel(DetailLevel.Tax);
        calendar.add(Calendar.DATE,-2);
        request.setDocDate(calendar.getTime());
        
        request.setDocCode("ApplyPaymentTest" +date.getTime());
        request.setCommit(true);
        GetTaxResult result = port.getTax(request);

        Assert.assertEquals(com.avalara.avatax.services.tax.SeverityLevel.Success, result.getResultCode());

        Calendar paymentDate =Calendar.getInstance();
        ApplyPaymentRequest applyPaymentRequest = new ApplyPaymentRequest();
        applyPaymentRequest.setDocId(result.getDocId());
        applyPaymentRequest.setPaymentDate(paymentDate);
        applyPaymentRequest.setDocType(DocumentType.SalesInvoice);
        ApplyPaymentResult applyPaymentResult = port.applyPayment(applyPaymentRequest);

        if(com.avalara.avatax.services.tax.SeverityLevel.Warning.equals(applyPaymentResult.getResultCode()))
        {
            Assert.assertEquals("ApplyPaymentWarning",applyPaymentResult.getMessages().getMessage(0).getName());
        }
        else
        {
            Assert.assertEquals(com.avalara.avatax.services.tax.SeverityLevel.Success,applyPaymentResult.getResultCode());
        }

    }

    public void fourTwelveChangesTest() throws RemoteException
    {
        Date date = new Date();
        String docCode = "CommitTest " + Long.toString(date.getTime());
        GetTaxRequest taxRequest = CreateTaxRequestReloaded(docCode,1);
        taxRequest.setDetailLevel(DetailLevel.Tax);

        GetTaxResult taxResult = port.getTax(taxRequest);

        Assert.assertEquals(SeverityLevel.Success, taxResult.getResultCode());
        Assert.assertEquals(taxRequest.getLines().size(), taxResult.getTaxLines().size());

        TaxLine taxLine = taxResult.getTaxLines().getTaxLine(0);
        Assert.assertEquals(2, taxLine.getTaxDetails().size());

        PostTaxRequest postRequest = new PostTaxRequest();

        postRequest.setCompanyCode(taxRequest.getCompanyCode());
        postRequest.setDocType(taxRequest.getDocType());
        postRequest.setDocCode(taxRequest.getDocCode());
        postRequest.setDocDate(taxRequest.getDocDate());
        postRequest.setTotalAmount(taxResult.getTotalAmount());
        postRequest.setTotalTax(taxResult.getTotalTax());

        PostTaxResult postResult = port.postTax(postRequest);

        Assert.assertEquals(SeverityLevel.Success, postResult.getResultCode());

        CommitTaxRequest commitRequest = new CommitTaxRequest();
        commitRequest.setDocId(postResult.getDocId());
        commitRequest.setDocType(taxRequest.getDocType());
        String newDocCode = Long.toString(date.getTime());
        //overwritting the DocCode
        commitRequest.setDocCode(newDocCode);
        CommitTaxResult commitResult = port.commitTax(commitRequest);


        //Finally Cancel Tax and remove the entry from the system by calling new DocVoided
        CancelTaxRequest cancelRequest = new CancelTaxRequest();
        cancelRequest.setDocId(commitResult.getDocId());
        cancelRequest.setDocType(taxRequest.getDocType());
        cancelRequest.setCancelCode(CancelCode.DocVoided);

        CancelTaxResult cancelResult = port.cancelTax(cancelRequest);

        Assert.assertEquals(SeverityLevel.Success, cancelResult.getResultCode());


        // Check tax history
        GetTaxHistoryRequest taxHistoryRequest = new GetTaxHistoryRequest();
        taxHistoryRequest.setDocId(taxResult.getDocId());
        taxHistoryRequest.setDetailLevel(DetailLevel.Tax);
        GetTaxHistoryResult taxHistoryResult = port.getTaxHistory(taxHistoryRequest);

        Assert.assertEquals(SeverityLevel.Success, taxHistoryResult.getResultCode());
        GetTaxRequest historyTaxRequest = taxHistoryResult.getGetTaxRequest();
        Assert.assertNotNull(historyTaxRequest);
        Assert.assertEquals(taxRequest.getAddresses().size(), historyTaxRequest.getAddresses().size());
        Assert.assertEquals(taxRequest.getLines().size(), historyTaxRequest.getLines().size());
        GetTaxResult historyTaxResult = taxHistoryResult.getGetTaxResult();
        Assert.assertNotNull(historyTaxResult);
        Assert.assertEquals(taxRequest.getLines().size(), historyTaxResult.getTaxLines().size());
        Assert.assertEquals(SeverityLevel.Success, commitResult.getResultCode());

        //match the values

        //compare id DocCode has been overwritten
        Assert.assertEquals(historyTaxResult.getDocCode(),newDocCode );
        //compare DocStatus is Cancelled as we straight away cancelled the Doc
        //System.out.print("Printing the status :"+historyTaxResult.getDocStatus());
        //Match HashCode
        Assert.assertEquals(taxResult.getHashCode() ,taxHistoryResult.getGetTaxResult().getHashCode());
        //Printing the TaxName
        //System.out.print("Printing TaxName : " + historyTaxResult.getTaxLines().getTaxLine(0).getTaxDetails().getTaxDetail(0).getTaxName());



        //Finally Cancel Tax and remove the entry from the system by calling new DocVoided
        //Delete the document from the system
        cancelRequest = new CancelTaxRequest();
        cancelRequest.setDocId(commitResult.getDocId());
        cancelRequest.setDocType(taxRequest.getDocType());
        cancelRequest.setCancelCode(CancelCode.DocDeleted);

        cancelResult = port.cancelTax(cancelRequest);

        Assert.assertEquals(SeverityLevel.Success, cancelResult.getResultCode());
        
    }

    public void fourSeventeenChangesTest() throws RemoteException
    {
        GetTaxRequest request = CreateTaxRequest("AdjustTaxTest"+new Date());
        request.setCommit(true);
        GetTaxResult result = port.getTax(request);

        if (result.getMessages().size() > 0)
        {
            System.out.println(result);
        }
        Assert.assertEquals(SeverityLevel.Success, result.getResultCode());
        Assert.assertEquals(1, result.getVersion());
        Assert.assertEquals(false, result.getLocked());
        Assert.assertEquals(0, result.getAdjustmentReason());
        Assert.assertEquals("", result.getAdjustmentDescription());
        Assert.assertEquals(DocStatus.Committed, result.getDocStatus());
        Assert.assertEquals(1, result.getVersion());
        Assert.assertEquals(result.getDocDate(), result.getTaxDate());


        //***************
        Line line = new Line();
        int lineNo = 2;
        line.setNo(Integer.toString(lineNo));
        line.setQty(new BigDecimal(2));
        line.setAmount(new BigDecimal(100.00));
        //request.lines.add();
        request.getLines().add(line);

        GetTaxHistoryRequest a = new GetTaxHistoryRequest();

        AdjustTaxRequest adjustTaxRequest = new AdjustTaxRequest();
        adjustTaxRequest.setAdjustmentReason(4); // Price or Qty Adjustment
        adjustTaxRequest.setGetTaxRequest(request);
        adjustTaxRequest.setAdjustmentDescription("Testing Adjustment");
        AdjustTaxResult adjustTaxResult = port.adjustTax(adjustTaxRequest);
        //TaxSvcAssert.IsSuccess(adjustTaxResult);
        Assert.assertEquals(2, adjustTaxResult.getVersion());
        Assert.assertEquals(4, adjustTaxResult.getAdjustmentReason());
        Assert.assertEquals("Testing Adjustment", adjustTaxResult.getAdjustmentDescription());
        Assert.assertEquals(DocStatus.Committed, result.getDocStatus());
        System.out.println("Adjusted Version 2 DocId: "+ adjustTaxResult.getDocId());


        // Cancel the last adjustment
        CancelTaxRequest cancelRequest = new CancelTaxRequest();
        cancelRequest.setDocId(adjustTaxResult.getDocId());
        cancelRequest.setCancelCode(CancelCode.AdjustmentCancelled);
        CancelTaxResult cancelResult = port.cancelTax(cancelRequest);
        //TaxSvcAssert.IsSuccess(cancelResult);
        System.out.println("Cancelled Version 2 DocId: {0}"+ cancelResult.getDocId());

        // Check the version
        GetTaxHistoryRequest taxHistoryRequest = new GetTaxHistoryRequest();
        taxHistoryRequest.setDocId(cancelResult.getDocId());
        taxHistoryRequest.setDetailLevel(DetailLevel.Document);
        GetTaxHistoryResult taxHistoryResult = port.getTaxHistory(taxHistoryRequest);
        //TaxSvcAssert.IsSuccess(taxHistoryResult);
        Assert.assertEquals(1, taxHistoryResult.getGetTaxResult().getVersion());
        Assert.assertEquals(0, taxHistoryResult.getGetTaxResult().getAdjustmentReason());

    }

    public void testTaxOverrideHeader()
    {
        try
        {
            GetTaxRequest request = CreateTaxRequest("TaxOverrideTest"+new Date());

            TaxOverride taxOverride = new TaxOverride();
            taxOverride.setTaxOverrideType(TaxOverrideType.TaxAmount);
            taxOverride.setTaxAmount(new BigDecimal(5));
            taxOverride.setTaxDate(new Date());
            taxOverride.setReason("Return");
            request.setCurrencyCode("USD");
            request.setServiceMode(ServiceMode.Automatic);

            request.setTaxOverride(taxOverride);

            GetTaxResult result = port.getTax(request);

			if (result.getMessages().size() > 0)
			{
				System.out.println(result.getMessages().getMessage(0).getSummary());
			}
			Assert.assertEquals(SeverityLevel.Success, result.getResultCode());

			Assert.assertEquals(SeverityLevel.Success, result.getResultCode());
			Assert.assertEquals(new BigDecimal(5), result.getTotalTax());

            // Check tax history
            GetTaxHistoryRequest taxHistoryRequest = new GetTaxHistoryRequest();
            taxHistoryRequest.setDocId(result.getDocId());
            taxHistoryRequest.setDetailLevel(DetailLevel.Tax);
            GetTaxHistoryResult taxHistoryResult = port.getTaxHistory(taxHistoryRequest);

            Assert.assertEquals(SeverityLevel.Success, taxHistoryResult.getResultCode());
            compareHistory(request, result, taxHistoryResult);
        }
        catch (RemoteException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }


    public void testTaxOverrideLine()
    {

        try
        {
            GetTaxRequest request = CreateTaxRequestForTaxOverride("TaxOverrideLineTest"+new Date());
            request.setCurrencyCode("USD");
            request.setServiceMode(ServiceMode.Automatic);


            GetTaxResult result = port.getTax(request);
            Assert.assertEquals(SeverityLevel.Success, result.getResultCode());            
            Assert.assertEquals(result.getTotalTax(),new BigDecimal(5));
            Assert.assertEquals(result.getTotalTaxCalculated(),new BigDecimal(81));

            Assert.assertEquals(result.getTaxLines().getTaxLine(0).getTax(),new BigDecimal(5));
            Assert.assertEquals(result.getTaxLines().getTaxLine(0).getTaxCalculated(),new BigDecimal(81));

           // Assert.assertEquals(result.getTaxLines().getTaxLine(0).g);

            // Check tax history
            GetTaxHistoryRequest taxHistoryRequest = new GetTaxHistoryRequest();
            taxHistoryRequest.setDocId(result.getDocId());
            taxHistoryRequest.setDetailLevel(DetailLevel.Tax);
            GetTaxHistoryResult taxHistoryResult = port.getTaxHistory(taxHistoryRequest);

            Assert.assertEquals(SeverityLevel.Success, taxHistoryResult.getResultCode());
            compareHistory(request, result, taxHistoryResult);
        }
        catch (RemoteException e)
        {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    public void testGetTaxGlobal() throws RemoteException, SOAPException
    {
        GetTaxRequest request = CreateTaxRequest("testGetTaxGlobal");

        // Change the destination to Canada
        BaseAddress destAddress = request.getAddresses().getBaseAddress(1);
        destAddress.setLine1("5412 Imperial St");
        destAddress.setLine2("");
        destAddress.setCity("Burnaby");
        destAddress.setRegion("BC");
        destAddress.setPostalCode("V5J 1E6");
        destAddress.setCountry("CA");

        GetTaxResult result = port.getTax(request);

        Assert.assertEquals(SeverityLevel.Success, result.getResultCode());
        Assert.assertEquals(DocStatus.Saved, result.getDocStatus());
        Assert.assertFalse(result.isReconciled());
        Assert.assertEquals(new BigDecimal(1010), result.getTotalAmount());
        Assert.assertEquals(BigDecimal.valueOf(1212, 1), result.getTotalTax());

        // Look for new Country/State jurisdictions
        TaxLine taxLine = result.getTaxLines().getTaxLine(0);
        TaxDetail taxDetail = findTaxDetail(JurisdictionType.Country, "CA", taxLine.getTaxDetails().getTaxDetail());
        Assert.assertNotNull("Expected Country=CA detail", taxDetail);
        taxDetail = findTaxDetail(JurisdictionType.State, "BC", taxLine.getTaxDetails().getTaxDetail());
        Assert.assertNotNull("Expected State=BC detail", taxDetail);

        // Check tax history
        GetTaxHistoryRequest taxHistoryRequest = new GetTaxHistoryRequest();
        taxHistoryRequest.setDocId(result.getDocId());
        taxHistoryRequest.setDetailLevel(DetailLevel.Tax);
        GetTaxHistoryResult taxHistoryResult = port.getTaxHistory(taxHistoryRequest);

        Assert.assertEquals(SeverityLevel.Success, taxHistoryResult.getResultCode());
        compareHistory(request, result, taxHistoryResult);
    }

    private void compareHistory(GetTaxRequest taxRequest, GetTaxResult taxResult, GetTaxHistoryResult history)
        {
            GetTaxRequest historyRequest = history.getGetTaxRequest();
            GetTaxResult historyResult = history.getGetTaxResult();


            Assert.assertEquals(taxRequest.getCompanyCode(), historyRequest.getCompanyCode());
            assertBigDecimalEquals(taxRequest.getDiscount(), historyRequest.getDiscount());
            Assert.assertEquals(taxRequest.getDocCode(), historyRequest.getDocCode());
            compareDate(taxRequest.getDocDate(), historyRequest.getDocDate());
            Assert.assertEquals(taxRequest.getDocType(), historyRequest.getDocType());
            assertStringEquals(taxRequest.getExemptionNo(),historyRequest.getExemptionNo());
            Assert.assertEquals(taxRequest.getCustomerCode(),historyRequest.getCustomerCode());
            assertStringEquals(taxRequest.getCustomerUsageType(),historyRequest.getCustomerUsageType());
            assertBigDecimalEquals(taxRequest.getExchangeRate(), historyRequest.getExchangeRate());
            compareDate(taxRequest.getExchangeRateEffDate(),historyRequest.getExchangeRateEffDate());

            Assert.assertEquals(taxResult.getDocId(), historyResult.getDocId());
            Assert.assertEquals(taxResult.getDocCode(),historyResult.getDocCode());
            Assert.assertEquals(taxResult.getDocDate(), historyResult.getDocDate());
            Assert.assertEquals(taxResult.getDocType(), historyResult.getDocType());
            Assert.assertEquals(taxResult.getDocStatus(), historyResult.getDocStatus());
            Assert.assertEquals(taxResult.getTimestamp().getTime().toString(), historyResult.getTimestamp().getTime().toString());
            Assert.assertEquals(taxResult.getTotalAmount(), historyResult.getTotalAmount());
            Assert.assertEquals(taxResult.getTotalTaxable(), historyResult.getTotalTaxable());
            Assert.assertEquals(taxResult.getTotalTax(), historyResult.getTotalTax());
            Assert.assertEquals(taxResult.getTotalTaxCalculated(), historyResult.getTotalTaxCalculated());
             
            Assert.assertEquals(taxRequest.getLines().getLine().length, historyRequest.getLines().getLine().length);
            for (int i=0;i<taxRequest.getLines().getLine().length;i++)
            {
                Line requestLine =taxRequest.getLines().getLine(i);

                Line historyLine = historyRequest.getLines().getLine(requestLine.getNo());

                Assert.assertNotNull(historyLine);
                Assert.assertEquals(requestLine.getNo(), historyLine.getNo());
                Assert.assertEquals(requestLine.getAmount(), historyLine.getAmount());
                assertStringEquals(requestLine.getItemCode(),historyLine.getItemCode());
            }

            //TaxResult.TaxSummary
            for(int i=0;i<taxResult.getTaxSummary().getTaxDetail().length;i++)
            {
                TaxDetail resultTaxDetail=taxResult.getTaxSummary().getTaxDetail(i);

                TaxDetail historyResultTaxDetail= findTaxDetail(resultTaxDetail.getJurisType(), resultTaxDetail.getJurisCode(), historyResult.getTaxSummary().getTaxDetail());
                Assert.assertNotNull(historyResultTaxDetail);

                Assert.assertEquals(resultTaxDetail.getCountry(), historyResultTaxDetail.getCountry());
                Assert.assertEquals(resultTaxDetail.getRegion(), historyResultTaxDetail.getRegion());
                Assert.assertEquals(resultTaxDetail.getJurisType(), historyResultTaxDetail.getJurisType());
                Assert.assertEquals(resultTaxDetail.getJurisCode(), historyResultTaxDetail.getJurisCode());
                Assert.assertEquals(resultTaxDetail.getTaxType(), historyResultTaxDetail.getTaxType());
                Assert.assertEquals(resultTaxDetail.getBase(), historyResultTaxDetail.getBase());
                Assert.assertEquals(resultTaxDetail.getTaxable(), historyResultTaxDetail.getTaxable());
                Assert.assertEquals(resultTaxDetail.getRate(), historyResultTaxDetail.getRate());
                Assert.assertEquals(resultTaxDetail.getTax(), historyResultTaxDetail.getTax());
                Assert.assertEquals(resultTaxDetail.getTaxCalculated(), historyResultTaxDetail.getTaxCalculated());
                Assert.assertEquals(resultTaxDetail.getNonTaxable(), historyResultTaxDetail.getNonTaxable());
                Assert.assertEquals(resultTaxDetail.getExemption(), historyResultTaxDetail.getExemption());
                Assert.assertEquals(resultTaxDetail.getJurisName(), historyResultTaxDetail.getJurisName());
                Assert.assertEquals(resultTaxDetail.getTaxName(), historyResultTaxDetail.getTaxName());
                Assert.assertEquals(resultTaxDetail.getTaxAuthorityType(), historyResultTaxDetail.getTaxAuthorityType());
                Assert.assertEquals(resultTaxDetail.getTaxGroup(), historyResultTaxDetail.getTaxGroup());

            }

            //GetTaxResult.TaxLine
            Assert.assertEquals(taxResult.getTaxLines().getTaxLine().length, historyResult.getTaxLines().getTaxLine().length);
            for (int i=0;i<taxResult.getTaxLines().getTaxLine().length;i++)
            {

                TaxLine resultLine=taxResult.getTaxLines().getTaxLine(i);

                TaxLine historyResultLine = historyResult.getTaxLines().getTaxLine(resultLine.getNo());
                Assert.assertNotNull(historyResultLine);

                Assert.assertEquals(resultLine.getNo(), historyResultLine.getNo());
                Assert.assertEquals(resultLine.getTaxable(), historyResultLine.getTaxable());                
                Assert.assertEquals(resultLine.getRate(), historyResultLine.getRate());
                Assert.assertEquals(resultLine.getTax(), historyResultLine.getTax());
                Assert.assertEquals(resultLine.getTaxCode(), historyResultLine.getTaxCode());
                Assert.assertEquals(resultLine.getTaxCalculated(),historyResultLine.getTaxCalculated());
                Assert.assertEquals(resultLine.getReportingDate(),historyResultLine.getReportingDate());
                Assert.assertEquals(resultLine.getAccountingMethod(),historyResultLine.getAccountingMethod());

                // TODO: Addresses
                

                // Line details
                Assert.assertEquals(resultLine.getTaxDetails().getTaxDetail().length, historyResultLine.getTaxDetails().getTaxDetail().length);
                for (int j=0;j < resultLine.getTaxDetails().getTaxDetail().length;j++)
                {
                     TaxDetail resultDetail =  resultLine.getTaxDetails().getTaxDetail(j);

                    TaxDetail historyDetail = findTaxDetail(resultDetail.getJurisType(), resultDetail.getJurisCode(), historyResultLine.getTaxDetails().getTaxDetail());
                    Assert.assertNotNull(historyDetail);

                    Assert.assertEquals(resultDetail.getTaxType(), historyDetail.getTaxType());
                    Assert.assertEquals(resultDetail.getBase(), historyDetail.getBase());
                    Assert.assertEquals(resultDetail.getJurisCode(), historyDetail.getJurisCode());
                    Assert.assertEquals(resultDetail.getJurisType(), historyDetail.getJurisType());
                    Assert.assertEquals(resultDetail.getRate(), historyDetail.getRate());
                    Assert.assertEquals(resultDetail.getTax(), historyDetail.getTax());
                }
            }
        }


      private TaxDetail findTaxDetail(JurisdictionType jurisdictionType, String jurisdictionCode, TaxDetail[] taxDetails)
        {
            TaxDetail match = null;

            for (int i=0;i<taxDetails.length;i++)
            {
                 TaxDetail detail = taxDetails[i];
                if (detail.getJurisType().equals(jurisdictionType) && detail.getJurisCode().equals(jurisdictionCode))
                {
                    match = detail;
                    break;
                }
            }

            return match;
        }
      private void compareDate(Date dateExpected, Date dateActual)
       {
           Calendar calExpected=Calendar.getInstance();
           calExpected.setTime(dateExpected);

           Calendar calActual=Calendar.getInstance();
           calActual.setTime(dateActual);

           String strExpected= calExpected.get(Calendar.DATE) + ""+ calExpected.get(Calendar.MONTH)+ "" + calExpected.get(Calendar.YEAR) + "";
           String strActual= calActual.get(Calendar.DATE) + "" + calActual.get(Calendar.MONTH) + "" +  calActual.get(Calendar.YEAR) + "";

           Assert.assertEquals(strExpected,strActual);
       }
    

    /*   This test case is no longer valid now
    public void testReconcile() throws RemoteException
    {
        ReconcileTaxHistoryRequest request = new ReconcileTaxHistoryRequest();
        request.setCompanyCode("DEFAULT");
        request.setReconciled(false);
        ReconcileTaxHistoryResult result = port.reconcileTaxHistory(request);
        GetTaxResult taxResults[] = result.getGetTaxResults().getGetTaxResult();
        Assert.assertEquals(SeverityLevel.Success, result.getResultCode());

        do
        {
            for (int ii = 0; taxResults != null && ii < taxResults.length; ii++)
            {
                GetTaxResult taxResult = taxResults[ii];
                Assert.assertEquals(false, taxResult.isReconciled());

                if (taxResult.getDocStatus() == DocStatus.Saved)
                {
                    // Cancel it
                    CancelTaxRequest cancelRequest = new CancelTaxRequest();
                    cancelRequest.setDocId(taxResult.getDocId());
                    cancelRequest.setCancelCode(CancelCode.DocDeleted);
                    CancelTaxResult cancelResult = port.cancelTax(cancelRequest);
                    Assert.assertEquals(SeverityLevel.Success, cancelResult.getResultCode());
                }
                else if (taxResult.getDocStatus() == DocStatus.Posted)
                {
                    // Commit it
                    CommitTaxRequest commitRequest = new CommitTaxRequest();
                    commitRequest.setDocId(taxResult.getDocId());
                    commitRequest.setDocType(DocumentType.SalesInvoice);
                    CommitTaxResult commitResult = port.commitTax(commitRequest);
                    Assert.assertEquals(SeverityLevel.Success, commitResult.getResultCode());
                }
                else
                {
                    Assert.assertEquals(DocStatus.Committed, taxResult.getDocStatus());
                }

            }

            request.setLastDocId(result.getLastDocId());
            result = port.reconcileTaxHistory(request);
            Assert.assertEquals(SeverityLevel.Success, result.getResultCode());
            taxResults = result.getGetTaxResults().getGetTaxResult();
        }
        while (taxResults != null && taxResults.length > 0);

        // Mark them all reconciled
        request.setReconciled(true);
        result = port.reconcileTaxHistory(request);
        Assert.assertEquals(SeverityLevel.Success, result.getResultCode());

        // Now verify they have all been reconciled
        request.setLastDocId("0");
        result = port.reconcileTaxHistory(request);

        Assert.assertEquals(SeverityLevel.Success, result.getResultCode());
        taxResults = result.getGetTaxResults().getGetTaxResult();
        Assert.assertTrue(taxResults == null || taxResults.length == 0);
    }   */

    //
    // Private methods
    //

    private GetTaxRequest CreateTaxRequest(String docCode)
    {
        return CreateTaxRequest(docCode, 1);
    }

    private GetTaxRequest CreateTaxRequestForTaxOverride(String docCode)
    {
        GetTaxRequest request = new GetTaxRequest();

        request.setCompanyCode("DEFAULT");
        request.setDocCode(docCode);
        request.setDocType(DocumentType.SalesInvoice);
        request.setDocDate(new Date());
        //request.setExemptionNo("");
        request.setCustomerCode("TaxSvcTest");
        //request.setPurchaserType("");
        request.setSalespersonCode("");
        request.setOriginCode("Origin");
        request.setDestinationCode("Dest");
        request.setDetailLevel(DetailLevel.Tax);

        ArrayOfBaseAddress addresses = new ArrayOfBaseAddress(2);
        BaseAddress origin = new BaseAddress();
        origin.setAddressCode("Origin");
        origin.setCity("Denver");
        origin.setRegion("CO");
        origin.setPostalCode("80216-1022");
        origin.setCountry("USA");
        addresses.add(origin);

        BaseAddress destination = new BaseAddress();
        destination.setAddressCode("Dest");
        destination.setLine1("11051 S Parker Rd");
        destination.setCity("Parker");
        destination.setRegion("CO");
        destination.setPostalCode("80134-7441");
        destination.setCountry("USA");
        addresses.add(destination);

        request.setAddresses(addresses);
        int lineCount = 1;
        ArrayOfLine lines = new ArrayOfLine(lineCount);
        Line line;

        line = new Line();
        int lineNo = 1;
        line.setNo(Integer.toString(lineNo));
        line.setQty(new BigDecimal(1));
        line.setAmount(new BigDecimal(1000));
        lines.add(line);

        TaxOverride taxOverride = new TaxOverride();
        taxOverride.setTaxOverrideType(TaxOverrideType.TaxAmount);
        taxOverride.setTaxAmount(new BigDecimal(5));
        taxOverride.setTaxDate(new Date());
        taxOverride.setReason("Return");

        line.setTaxOverride(taxOverride);

        request.setExchangeRate(new BigDecimal(1));
        request.setExchangeRateEffDate(request.getDocDate());
        request.setPaymentDate(new Date());
        request.setLines(lines);

        return request;
    }

    private GetTaxRequest CreateTaxRequest(String docCode, int lineCount)
    {
        GetTaxRequest request = new GetTaxRequest();

        request.setCompanyCode("DEFAULT");                                
        request.setDocCode(docCode);
        request.setDocType(DocumentType.SalesInvoice);
        request.setDocDate(new Date());
        //request.setExemptionNo("");
        request.setCustomerCode("TaxSvcTest");
        //request.setPurchaserType("");
        request.setSalespersonCode("");
        request.setOriginCode("Origin");
        request.setDestinationCode("Dest");
        request.setDetailLevel(DetailLevel.Tax);
        

        request.setExchangeRate(new BigDecimal(1.5));
        request.setExchangeRateEffDate(new Date("1/1/2008"));

        ArrayOfBaseAddress addresses = new ArrayOfBaseAddress(2);
        BaseAddress origin = new BaseAddress();
        origin.setAddressCode("Origin");
        origin.setLine1("Avalara");
        origin.setLine2("900 Winslow Way");
        origin.setLine3("Suite 100");
        origin.setCity("Bainbridge Island");
        origin.setRegion("WA");
        origin.setPostalCode("98110");
        origin.setCountry("USA");
        addresses.add(origin);

        BaseAddress destination = new BaseAddress();
        destination.setAddressCode("Dest");
        destination.setLine1("3130 Elliott");
        destination.setCity("Seattle");
        destination.setRegion("WA");
        destination.setPostalCode("98121");
        destination.setCountry("USA");
        addresses.add(destination);

        request.setAddresses(addresses);

        ArrayOfLine lines = new ArrayOfLine(lineCount+1);
        Line line;

        int baseLineNo = 1000;

        for (int ii = 0; ii < lineCount; ii++)
        {
            line = new Line();
            int lineNo = baseLineNo + ii + 1;
            line.setNo(Integer.toString(lineNo));
            line.setItemCode("Item001");
            line.setQty(new BigDecimal(1));
            line.setAmount(new BigDecimal(1000.00));
            lines.add(line);
        }

        line = new Line();
        int lineNo = baseLineNo + lineCount + 1;
        line.setNo(Integer.toString(lineNo));
        line.setItemCode("FR");
        line.setQty(new BigDecimal(1));
        line.setAmount(new BigDecimal(10.00));
        lines.add(line);

        request.setLines(lines);

         request.setPaymentDate(new Date());

        return request;
    }
    //This function is used to Test the 4.12 changes
    //This will add few more parameters which addresses 4.12
    private GetTaxRequest CreateTaxRequestReloaded(String docCode, int lineCount)
    {
        GetTaxRequest request = new GetTaxRequest();

        request.setCompanyCode("DEFAULT");
        request.setDocCode(docCode);
        request.setDocType(DocumentType.SalesInvoice);
        request.setDocDate(new Date());
        //request.setExemptionNo("");
        request.setCustomerCode("TaxSvcTest");
        //request.setPurchaserType("");
        request.setSalespersonCode("");
        request.setOriginCode("Origin");
        request.setDestinationCode("Dest");
        request.setDetailLevel(DetailLevel.Document);
        //added in 4.12 release
        request.setLocationCode("Test LocationCode");
        request.setHashCode(007);

        ArrayOfBaseAddress addresses = new ArrayOfBaseAddress(2);
        BaseAddress origin = new BaseAddress();
        origin.setAddressCode("Origin");
        origin.setLine1("Avalara");
        origin.setLine2("900 Winslow Way");
        origin.setLine3("Suite 100");
        origin.setCity("Bainbridge Island");
        origin.setRegion("WA");
        origin.setPostalCode("98110");
        origin.setCountry("USA");
        addresses.add(origin);

        BaseAddress destination = new BaseAddress();
        destination.setAddressCode("Dest");
        destination.setLine1("3130 Elliott");
        destination.setCity("Seattle");
        destination.setRegion("WA");
        destination.setPostalCode("98121");
        destination.setCountry("USA");
        addresses.add(destination);

        request.setAddresses(addresses);

        ArrayOfLine lines = new ArrayOfLine(lineCount+1);
        Line line;

        int baseLineNo = 1000;

        for (int ii = 0; ii < lineCount; ii++)
        {
            line = new Line();
            int lineNo = baseLineNo + ii + 1;
            line.setNo(Integer.toString(lineNo));
            line.setItemCode("Item001");
            line.setQty(new BigDecimal(1));
            line.setAmount(new BigDecimal(1000.00));
            //added in 4.12 release
            line.setCustomerUsageType("G");
            line.setDescription("Sample Description");
            lines.add(line);
        }

        line = new Line();
        int lineNo = baseLineNo + lineCount + 1;
        line.setNo(Integer.toString(lineNo));
        line.setItemCode("FR");
        line.setQty(new BigDecimal(1));
        line.setAmount(new BigDecimal(10.00));
        lines.add(line);

        request.setLines(lines);

        return request;
    }

    /**
     * Asserts equality of strings, treating null as an empty string.
     * @param s1
     * @param s2
     */
    private void assertStringEquals(String s1, String s2)
    {
        if (s1 == null)
        {
            s1 = "";
        }
        if (s2 == null)
        {
            s2 = "";
        }
        Assert.assertEquals(s1, s2);
    }

    private void assertBigDecimalEquals(BigDecimal d1, BigDecimal d2)
    {
        if (d1 == null)
        {
            d1 = new BigDecimal(0);
        }
        if (d2 == null)
        {
            d2 = new BigDecimal(0);
        }
        Assert.assertEquals(d1.stripTrailingZeros(), d2.stripTrailingZeros());
    }
}